#include "windef.h"
#include "classes.h"
#include <stdio.h>
#include <math.h>
#include <string.h>

CB3ByteString::CB3ByteString() {

	m_i32AllocSize = 0;
	m_i32BufferSize = 0;

	m_bfBuffer = NULL;

}

CB3ByteString::CB3ByteString(I32 i32Size) {

	m_i32AllocSize = 0;
	m_i32BufferSize = 0;

	m_bfBuffer = NULL;

	resizeBuffer(i32Size);

}

CB3ByteString::CB3ByteString(const STR8 s8Arg) {

	m_i32AllocSize = 0;
	m_i32BufferSize = 0;

	m_bfBuffer = NULL;

	assign(s8Arg);

}

CB3ByteString::CB3ByteString(const BUF bfBuf, I32 i32Size) {

	m_i32AllocSize = 0;
	m_i32BufferSize = 0;

	m_bfBuffer = NULL;

	assign(bfBuf, i32Size);

}

CB3ByteString::CB3ByteString(const IB3ByteString *pbsString) {

	m_i32AllocSize = 0;
	m_i32BufferSize = 0;

	m_bfBuffer = NULL;

	assign(pbsString);

}

CB3ByteString::CB3ByteString(const CB3ByteString &bsString) {

	m_i32AllocSize = 0;
	m_i32BufferSize = 0;

	m_bfBuffer = NULL;

	assign(&bsString);

}

CB3ByteString::~CB3ByteString() {

	clear();

}

CB3ByteString & CB3ByteString::operator=(const CB3ByteString& bsArg) {

	if (&bsArg!=this) {

		clear();

		assign(&bsArg);

	}

	return *this;

}

CB3ByteString & CB3ByteString::operator=(const STR8 s8Arg) {

	assign(s8Arg);

	return *this;

}

CB3ByteString & CB3ByteString::operator+=(UI8 u8Arg) {

	append(u8Arg);

	return *this;

}

CB3ByteString & CB3ByteString::operator+=(const STR8 s8Arg) {

	append(s8Arg);

	return *this;

}

CB3ByteString & CB3ByteString::operator+=(const CB3ByteString& bsArg) {

	append(bsArg);

	return *this;

}

bool CB3ByteString::clear() {

	if (m_i32AllocSize > 0) {

		B3FREE_MEMORY(m_bfBuffer);
		m_bfBuffer = NULL;
	
	}

	m_i32AllocSize = 0;
	m_i32BufferSize = 0;

	return true;

}

bool CB3ByteString::assign(const BUF bfNewString, I32 i32StringSize) {

	if (bfNewString == NULL || i32StringSize < 0) {
		return false;
	}

	if (!resizeBuffer(i32StringSize)) {
		return false;
	}

	if (i32StringSize == 0) {
		return true;
	}

	memcpy(m_bfBuffer, bfNewString, i32StringSize);

	return true;

}

bool CB3ByteString::assign(UI8 u8Byte) {

	if (!resizeBuffer(1)) {
		return false;
	}

	m_bfBuffer[0] = u8Byte;

	return true;

}

bool CB3ByteString::assign(const STR8 s8NewString) {

	if (s8NewString == NULL) {

		clear();

		return true;

	}

	return assign((BUF)s8NewString, (I32)strlen(s8NewString));

}

bool CB3ByteString::assign(const IB3ByteString *ibsNewString) {

	if (ibsNewString == NULL) {
		return false;
	}

	if (ibsNewString == this) {
		return true;
	}

	return assign(ibsNewString->getBuffer(), ibsNewString->getSize());

}

bool CB3ByteString::append(const BUF bfAppend,I32 i32Size) {

	if (bfAppend == NULL || i32Size<1) {
		return false;
	}

	I32 i32Last = getLastBufferIndex() + 1;

	resizeBuffer(m_i32BufferSize + i32Size);

	memcpy(m_bfBuffer + i32Last, bfAppend, i32Size);

	return true;

}

bool CB3ByteString::append(const STR8 s8Append) {

	if (s8Append == NULL) {
		return false;
	}

	return append((BUF)s8Append, (I32)strlen(s8Append));

}

bool CB3ByteString::append(UI8 u8Append) {

	if (!resizeBuffer(m_i32BufferSize + 1)) {
		return false;
	}

	m_bfBuffer[getLastBufferIndex()] = u8Append;

	return true;

}

bool CB3ByteString::append(const CB3ByteString& bsAppend) {

	return append(&bsAppend);

}

bool CB3ByteString::append(const IB3ByteString *ibsAppend) {

	if (ibsAppend == NULL) {
		return false;
	}

	if (ibsAppend == this) {

		BUF bfTmp = B3ALLOC_MEMORY(m_i32BufferSize);

		memcpy(bfTmp, m_bfBuffer, m_i32BufferSize);

		bool bResult = append(bfTmp, m_i32BufferSize);

		B3FREE_MEMORY(bfTmp);

		return bResult;

	}

	return append(ibsAppend->getBuffer(), ibsAppend->getSize());

}

bool CB3ByteString::del(I32 i32Index) {

	if (i32Index < 0 || i32Index > getLastBufferIndex()) {
		return false;
	}

	if (m_i32BufferSize < 2) {
		return clear();
	}

	if (i32Index == 0) {
		memmove(m_bfBuffer, m_bfBuffer + 1, m_i32BufferSize - 1);
	} else if (i32Index != getLastBufferIndex()) {
		memmove(m_bfBuffer + i32Index, m_bfBuffer + i32Index + 1, m_i32BufferSize - i32Index - 1);
	}

	m_i32BufferSize--;

	m_bfBuffer[getLastBufferIndex() + 1] = 0;

	return true;

}

bool CB3ByteString::del(I32 i32Index, I32 i32Length) {

	if (i32Index == 0 && i32Length == m_i32BufferSize) {
		return clear();
	}

	if (i32Index < 0 || i32Length < 1 || m_i32BufferSize < i32Index + i32Length) {
		return false;
	}

	if (i32Length == 1) {
		return del(i32Index);
	}

	if (i32Index + i32Length == m_i32BufferSize) {
		return resizeBuffer(m_i32BufferSize - i32Length);
	}

	if (i32Index == 0) {
		memmove(m_bfBuffer, m_bfBuffer + i32Length, m_i32BufferSize - i32Length);
	} else {
		memmove(m_bfBuffer + i32Index, m_bfBuffer + i32Index + i32Length, m_i32BufferSize - (i32Index + i32Length));
	}

	m_i32BufferSize -= i32Length;

	m_bfBuffer[getLastBufferIndex() + 1] = 0;

	return true;

}

I32 CB3ByteString::trim(UI8 u8Key) {

	if (m_i32BufferSize < 1) {
		return 0;
	}

	I32 i32Num = 0;

	while (m_i32BufferSize > 0 && m_bfBuffer[0] == u8Key) {

		del(0);

		i32Num++;

	}

	return i32Num;

}

bool CB3ByteString::insert(I32 i32Index, UI8 u8Byte) {

	if (i32Index < 0 || i32Index > getLastBufferIndex()) {
		return false;
	}

	if (m_i32BufferSize == 0) {
		return assign(u8Byte);
	}

	if (i32Index == getLastBufferIndex()) {
		return append(u8Byte);
	}

	I32 i32Last = getLastBufferIndex();

	if (!resizeBuffer(m_i32BufferSize + 1)) {
		return false;
	}

	if (i32Index == 0) {

		memmove(m_bfBuffer+1, m_bfBuffer, m_i32BufferSize - 1);

		m_bfBuffer[0] = u8Byte;

		return true;

	}

	memmove(m_bfBuffer + i32Index+ 1, m_bfBuffer + i32Index, m_i32BufferSize - i32Index -1);

	m_bfBuffer[i32Index] = u8Byte;

	return true;

}

bool CB3ByteString::insert(I32 i32Index, const BUF bfString, I32 i32Size) {

	if (i32Index < 0 || i32Index > getLastBufferIndex() ||
	    bfString == NULL || i32Size < 1) {
		return false;
	}

	if (m_i32BufferSize == 0) {
		return assign(bfString, i32Size);
	}

	if (!resizeBuffer(m_i32BufferSize + i32Size)) {
		return false;
	}

	if (i32Index == 0) {

		memmove(m_bfBuffer+i32Size, m_bfBuffer, m_i32BufferSize - i32Size);
		memmove(m_bfBuffer, bfString, i32Size);

		return true;

	}

	memmove(m_bfBuffer + i32Index + i32Size, m_bfBuffer + i32Index, m_i32BufferSize - i32Index - i32Size);

	memmove(m_bfBuffer + i32Index, bfString, i32Size);

	return true;

}

bool CB3ByteString::setByte(I32 i32Index, UI8 u8Val) {

	if (i32Index < 0 || i32Index > getLastBufferIndex()) {
		return false;
	}

	m_bfBuffer[i32Index] = u8Val;

	return true;

}

I32 CB3ByteString::search(I32 i32Start, UI8 u8Key) const {

	if (i32Start < 0 || i32Start >= m_i32BufferSize) {
		return -1;
	}

	for (int i = i32Start;i < m_i32BufferSize;i++) {
		if (m_bfBuffer[i] == u8Key) {
			return i;
		}
	}

	return -1;

}

I32 CB3ByteString::search(const UI8 u8Key) const {
	return search(0, u8Key);
}

I32 CB3ByteString::search(const STR8 s8Key) const {
	return search(0, s8Key);
}

I32 CB3ByteString::search(const BUF bfKey, I32 i32Size) const {
	return search(0, bfKey, i32Size);
}

I32 CB3ByteString::search(I32 i32Start, const STR8 s8Key) const {
	return search(i32Start, (BUF)s8Key, (I32)strlen(s8Key));
}

I32 CB3ByteString::search(I32 i32Start, const BUF bfKey, I32 i32Size) const {

	if (i32Start < 0 || i32Start > m_i32BufferSize - i32Size || bfKey == NULL ||
	    i32Size <1 || i32Size > m_i32BufferSize) {
		return -1;
	}

	I32 i32StrPos = i32Start;

	while (i32StrPos <= m_i32BufferSize - i32Size) {

		if (m_bfBuffer[i32StrPos] == bfKey[0]) {

			bool bFind = false;

			for (I32 i = 0;i < i32Size;i++) {
				if (m_bfBuffer[i32StrPos + i32Size -1] == bfKey[i]) {
					bFind = true;
				}
			}

			if (bFind) {

				I32 i = 0;

				while ((i < i32Size) && (m_bfBuffer[i32StrPos + i] == bfKey[i])) {
						i++;
				}

				if (i == i32Size) {
					return i32StrPos;
				}

				i32StrPos++;

			} else {
				i32StrPos += i32Size;
			}

		} else {
			i32StrPos++;
		}

	}

	return -1;

}

I32 CB3ByteString::searchN(const BUF bfKey, I32 i32Size) const {
	return searchN(0, bfKey, i32Size);
}

I32 CB3ByteString::searchN(I32 i32Start, const BUF bfKey, I32 i32Size) const {

	if (i32Start < 0 || i32Start > m_i32BufferSize - i32Size || bfKey == NULL ||
	    i32Size <1 || i32Size > m_i32BufferSize) {
		return 0;
	}

	I32 i32Pos = i32Start, i32N = 0;

	while (i32Pos != -1) {

		i32Pos = search(i32Pos, bfKey, i32Size);

		if (i32Pos != -1) {

			i32N++;
			i32Pos++;

		}

	}

	return i32N;

}

IB3ByteStrings * CB3ByteString::split(const STR8 s8Key) const {
	return split((BUF)s8Key, (I32)strlen(s8Key));
}

IB3ByteStrings * CB3ByteString::split(const BUF bfKey, I32 i32Size) const {

	if (bfKey == NULL || i32Size < 1 || m_i32BufferSize < i32Size) {
		return NULL;
	}

	I32 i32Tokens = searchN(bfKey, i32Size) + 1;

	IB3ByteStrings *pbssToken = new CB3ByteStringVector();

	I32 i32End, i32Index = 0;

	for (I32 i = 0;i < i32Tokens;i++) {

		if (i == i32Tokens - 1) {
			i32End = m_i32BufferSize;
		} else {
			i32End = search(i32Index, bfKey, i32Size);
		}

		CB3ByteString *pbsTmp = new CB3ByteString(m_bfBuffer + i32Index, i32End - i32Index);
		pbssToken->add(pbsTmp);
		delete pbsTmp;


		i32Index = i32End + i32Size;

	}

	return pbssToken;

}

bool CB3ByteString::endWith(const BUF bfKey, I32 i32Key) const {

	if (bfKey == NULL || i32Key < 1 || m_i32BufferSize < i32Key) {
		return false;
	}

	for (I32 i = 0;i < i32Key;i++) {
		if (m_bfBuffer[getLastBufferIndex() - i] != bfKey[i32Key - 1 -i]) {
			return false;
		}
	}

	return true;

}

I32 CB3ByteString::replace(const STR8 s8Key, const STR8 s8Word) {
	return replace((BUF)s8Key, (I32)strlen(s8Key), (BUF)s8Word, (I32)strlen(s8Word));
}

I32 CB3ByteString::replace(const BUF bfKey, I32 i32Key, const BUF bfWord, I32 i32Word) {

	if (bfKey == NULL || i32Key < 0 || bfWord == NULL || i32Word < 0) {
		return 0;
	}

	I32 i32Num = searchN(bfKey, i32Key);

	if (i32Num < 1) {
		return 0;
	}

	IB3ByteStrings *bssTmp = split(bfKey, i32Key);

	bool bEnd = endWith((BUF)bfWord, i32Word);

	clear();

	for (I32 i = 0;i < bssTmp->getSize();i++) {

		append(bssTmp->getString(i));

		if (!(i == bssTmp->getSize() - 1 && !bEnd)) {
			append(bfWord, i32Word);
		}

	}

	delete bssTmp;

	return i32Num;

}

bool CB3ByteString::replace(I32 i32Index, I32 i32Length, const BUF bfValue, I32 i32ValLength) {

	if (i32Index < 0 || i32Length < 0 || m_i32BufferSize < i32Index + i32Length) {
		return false;
	}

	if (i32ValLength == 0) {
		return del(i32Index, i32Length);
	}

	if (bfValue == NULL) {
		return false;
	}

	CB3ByteString bsTmp = CB3ByteString(this);

	if (i32Index + i32Length == m_i32BufferSize) {
		if (!(bsTmp.del(i32Index, i32Length) && bsTmp.append(bfValue, i32ValLength))) {
			return false;
		}
	} else {
		if (!(bsTmp.del(i32Index, i32Length) && bsTmp.insert(i32Index, bfValue, i32ValLength))) {
			return false;
		}
	}

	return assign(&bsTmp);

}

IB3ByteString * CB3ByteString::substring(I32 i32Index, I32 i32Length) const {

	if (i32Index < 0 || i32Length < 1 || i32Index + i32Length > m_i32BufferSize) {
		return NULL;
	}

	CB3ByteString *pbsReturn = new CB3ByteString(i32Length);

	if (!pbsReturn->assign(getBuffer() + i32Index, i32Length)) {

		delete pbsReturn;

		return NULL;
	}

	return pbsReturn;

}

I32 CB3ByteString::collect(UI8 u8Byte) {

	if (m_i32BufferSize < 2) {
		return 0;
	}

	I32 i32Index = 0;
	I32 i32Del = 0;
	do {

		if (getByte(i32Index) == u8Byte && i32Index < getLastBufferIndex()) {

			i32Index++;

			while (getByte(i32Index) == u8Byte && i32Index <= getLastBufferIndex()) {

				del(i32Index);

				i32Del++;

			}
		}

		i32Index++;

	} while(i32Index < getLastBufferIndex());

	return i32Del;

}

I32 CB3ByteString::getSize() const { return m_i32BufferSize; }

const BUF CB3ByteString::getBuffer() const { return (const BUF)m_bfBuffer; }

const STR8 CB3ByteString::c_str() const { return (const STR8)m_bfBuffer; }

bool CB3ByteString::resizeBuffer(I32 i32Size) {

	if (i32Size < 0) {
		return false;
	}

	if (i32Size == 0) {
		return clear();
	}

	if (i32Size == m_i32BufferSize) {
		return true;
	}

	if (m_i32BufferSize == 0) {

		if (!allocBuffer(i32Size + 1)) {
			return false;
		}

		m_i32BufferSize = i32Size;
		m_bfBuffer[getLastBufferIndex() +1] = 0;

		return true;

	}

	if (i32Size < m_i32AllocSize && i32Size > m_i32AllocSize / 2) {

		m_i32BufferSize = i32Size;
		m_bfBuffer[getLastBufferIndex() +1] = 0;

		return true;

	}

	I32 i32CopySize = i32Size;

	if (i32CopySize > m_i32BufferSize) {
		i32CopySize = m_i32BufferSize;
	}

	BUF bfTmp = B3ALLOC_MEMORY(i32CopySize);

	memcpy(bfTmp, m_bfBuffer, i32CopySize);

	if (!allocBuffer(i32Size+1)) {

		B3FREE_MEMORY(bfTmp);

		return false;

	}

	memcpy(m_bfBuffer, bfTmp, i32CopySize);

	m_i32BufferSize = i32Size;

	B3FREE_MEMORY(bfTmp);

	m_bfBuffer[getLastBufferIndex() +1] = 0;

	return true;

}

bool CB3ByteString::allocBuffer(I32 i32Size) {

	if (i32Size < 0) {
		return false;
	}

	if (i32Size == 0) {
		return clear();
	}

	if (i32Size < m_i32AllocSize) {

		BUF bfTmp = B3ALLOC_MEMORY(i32Size);

		if (bfTmp == NULL) {
			return false;
		}

		if (m_i32AllocSize > 0) {
			B3FREE_MEMORY(m_bfBuffer);
		}

		m_bfBuffer = bfTmp;
		m_i32AllocSize = i32Size;

		return true;

	}

	if (i32Size == m_i32AllocSize) {
		return true;
	}

	if (i32Size < m_i32AllocSize * 2) {

		BUF bfTmp = B3ALLOC_MEMORY(m_i32AllocSize * 2);

		if (m_i32AllocSize > 0) {
			B3FREE_MEMORY(m_bfBuffer);
		}

		m_bfBuffer = bfTmp;
		m_i32AllocSize = m_i32AllocSize * 2;

		return true;

	} else {

		BUF bfTmp = B3ALLOC_MEMORY(i32Size);

		if (m_i32AllocSize > 0) {
			B3FREE_MEMORY(m_bfBuffer);
		}

		m_bfBuffer = bfTmp;
		m_i32AllocSize = i32Size;

		return true;

	}

}

I32 CB3ByteString::addVal(I32 i32Min, I32 i32Max, I32 i32Add) {

	if (i32Min > 255 || i32Max < 0 || i32Add == 0) {
		return 0;
	}

	I32 i32Num = 0;

	for (int i = 0;i <= getLastBufferIndex();i++) {

		I32 i32Val = getByte(i);

		if (i32Val >= i32Min && i32Val <= i32Max) {

			i32Val += i32Add;

			if (i32Val < 0) {
				i32Val = 0;
			}

			if (i32Val > 255) {
				i32Val = 255;
			}

			setByte(i, (UI8)i32Val);

			i32Num++;

		}

	}

	return i32Num;

}

I32 CB3ByteString::getLastBufferIndex() const { return m_i32BufferSize - 1; }

UI8 CB3ByteString::getByte(I32 i32Index) const {

	if (i32Index <0 || i32Index >= m_i32BufferSize) {
		return 0;
	}

	return m_bfBuffer[i32Index];

}

bool CB3ByteString::loadFile(const STR8 s8FilePath) {

	if (s8FilePath == NULL) {
		return false;
	}

	FILE *fp = fopen(s8FilePath, "rb");

	if (fp == NULL) {
		return false;
	}

	fseek(fp, 0, SEEK_END);

	I32 i32Len = ftell(fp) + 1;

	rewind(fp);

	if (i32Len < 1) {

		fclose(fp);

		clear();

		return true;

	}

	BUF bfTmp = B3ALLOC_MEMORY(i32Len);

	fread(bfTmp, 1, i32Len, fp);

	fclose(fp);

	assign(bfTmp, i32Len);

	B3FREE_MEMORY(bfTmp);

	return true;

}

bool CB3ByteString::saveFile(const STR8 s8FilePath) const {

	if (s8FilePath == NULL) {
		return false;
	}

	FILE *fp = fopen(s8FilePath, "wb");

	if (fp == NULL) {
		return false;
	}

	fwrite(getBuffer(), 1, m_i32BufferSize, fp);

	fclose(fp);

	return true;

}
