#pragma once
#include "windef.h"
#include <stdio.h>

class IB3ByteStrings;

class IB3ByteString {

public:

	virtual bool clear() = 0;

	virtual bool assign(const BUF, I32) = 0;
	virtual bool assign(const STR8) = 0;
	virtual bool assign(UI8) = 0;
	virtual bool assign(const IB3ByteString *) =0;

	virtual bool append(const BUF, I32) = 0;
	virtual bool append(const STR8) = 0;
	virtual bool append(UI8) = 0;
	virtual bool append(const IB3ByteString *) = 0;

	virtual bool del(I32) = 0;
	virtual bool del(I32, I32) = 0;

	virtual bool insert(I32, UI8) = 0;
	virtual bool insert(I32, const BUF, I32) = 0;

	virtual I32 search(I32, UI8) const = 0;
	virtual I32 search(const BUF, I32) const = 0;
	virtual I32 search(I32, const BUF, I32) const = 0;

	virtual I32 searchN(const BUF, I32) const =0;
	virtual I32 searchN(I32, const BUF, I32) const =0;

	virtual IB3ByteStrings * split(const STR8) const = 0;
	virtual IB3ByteStrings * split(const BUF, I32) const = 0;

	virtual I32 getSize() const = 0;

	virtual UI8 getByte(I32) const = 0;

	virtual const BUF getBuffer() const = 0;
	virtual bool resizeBuffer(I32) = 0;
	virtual const STR8 c_str() const = 0;
	virtual bool setByte(I32, UI8) = 0;

	virtual bool loadFile(const STR8) = 0;
	virtual bool saveFile(const STR8) const = 0;

	virtual ~IB3ByteString() {}

};

class IB3ByteStrings {

public:

	virtual bool add(const IB3ByteString *) = 0;

	virtual IB3ByteString * getString(I32) const= 0;

	virtual I32 getSize() const =0;

	virtual bool clear() = 0;

	virtual ~IB3ByteStrings() {}

};

class CB3ByteString:public IB3ByteString {

public:

	CB3ByteString();
	CB3ByteString(I32);
	CB3ByteString(const STR8);
	CB3ByteString(const BUF, I32);
	CB3ByteString(const IB3ByteString *);
	CB3ByteString(const CB3ByteString &);

	~CB3ByteString();

	virtual CB3ByteString & operator=(const STR8);
	virtual CB3ByteString & operator=(const CB3ByteString&);
	virtual CB3ByteString & operator+=(const UI8);
	virtual CB3ByteString & operator+=(const STR8);
	virtual CB3ByteString & operator+=(const CB3ByteString&);

	virtual bool clear();

	virtual bool assign(UI8);
	virtual bool assign(const STR8);
	virtual bool assign(const BUF,I32);
	virtual bool assign(const IB3ByteString *);

	virtual bool append(UI8);
	virtual bool append(const STR8);
	virtual bool append(const BUF,I32);
	virtual bool append(const CB3ByteString&);
	virtual bool append(const IB3ByteString *);

	virtual bool del(I32);
	virtual bool del(I32, I32);

	virtual I32 trim(UI8);

	virtual bool insert(I32, UI8);
	virtual bool insert(I32, const BUF, I32);

	virtual bool setByte(I32, UI8);

	virtual I32 search(UI8) const;
	virtual I32 search(const STR8) const;
	virtual I32 search(const BUF, I32) const;
	virtual I32 search(I32, UI8) const;
	virtual I32 search(I32, const STR8) const;
	virtual I32 search(I32, const BUF, I32) const;

	virtual I32 searchN(const BUF, I32) const;
	virtual I32 searchN(I32, const BUF, I32) const;

	virtual IB3ByteString * substring(I32, I32) const;

	virtual I32 collect(UI8);

	virtual IB3ByteStrings * split(const STR8) const;
	virtual IB3ByteStrings * split(const BUF, I32) const;

	virtual bool endWith(const BUF, I32) const;

	virtual I32 replace(const STR8, const STR8);
	virtual I32 replace(const BUF, I32, const BUF, I32);
	virtual bool replace(I32, I32, const BUF, I32);

	virtual I32 addVal(I32, I32, I32);

	virtual I32 getSize() const;

	virtual UI8 getByte(I32) const;

	virtual const BUF getBuffer() const;
	virtual const STR8 c_str() const;

	virtual bool resizeBuffer(I32);

	virtual I32 getLastBufferIndex() const;

	virtual bool loadFile(const STR8);
	virtual bool saveFile(const STR8) const;

private:

	bool allocBuffer(I32);

	I32 m_i32AllocSize, m_i32BufferSize, m_i32STR8Size;
	BUF m_bfBuffer;

};


class CB3ByteStringVector:public IB3ByteStrings {

public:

	CB3ByteStringVector();
	~CB3ByteStringVector();

	virtual bool add(const IB3ByteString *);

	virtual I32 getSize() const;
	virtual IB3ByteString * getString(I32) const;

	virtual bool clear();

private:

	IB3ByteString **m_ppbsString;
	I32 m_i32Strings,m_i32AllocSize;

	bool resizeVector(I32);

};
