#pragma once
#include <windows.h>

typedef enum {

	pdt4BGR24 = 0x0101,
	pdtBGRX32 = 0x0104,
	pdtBMP    = 0x0180

} PixelDataType;

/* DIBSectionNX */
class CMDIBSection {

public:

	CMDIBSection();
	CMDIBSection(int iWidth, int iHeight);

	CMDIBSection(const CMDIBSection& disSrc);
	CMDIBSection(const CMDIBSection * lpdisSrc);

	bool getValid() const;
	bool Invalidate();

	virtual ~CMDIBSection();

	CMDIBSection& operator=(const CMDIBSection& disSrc);

	virtual bool assign(const CMDIBSection *lpbmpSrc);
	virtual bool assign(const HBITMAP hbmpSrc);

	virtual bool assign(const void * lpData, PixelDataType pdtType);

	virtual bool setPixels(const void * lpData, PixelDataType pdtType); 

	virtual void clear();
	virtual bool clear(int iWidth, int iHeight, DWORD dwValue = 0x00000000);

	void setClearRGB(DWORD dwRGB);
	void setClearBGR(DWORD dwBGR);

	virtual bool load(LPCTSTR lpszFilePath);
	virtual bool save(LPCTSTR lpszFilePath) const;

	virtual int getWidth() const;
	virtual int getHeight() const;

	virtual const BITMAPINFO * getInfo() const;
	virtual HDC getHDC();

	virtual DWORD getRGB(int iX, int iY) const;
	virtual DWORD getBGR(int iX, int iY) const;
	virtual DWORD getPixel(int iX, int iY) const;
	virtual BYTE getR(int iX, int iY) const;
	virtual BYTE getG(int iX, int iY) const;
	virtual BYTE getB(int iX, int iY) const;

	virtual bool isColorKey(int iX, int iY) const;

	virtual bool set4BGR24(int iX, int iY, int iWidth, int iHeight, const void *lpRGB);

	virtual bool setRGB(int iX, int iY, DWORD dwValue);
	virtual bool setPixel(int iX, int iY, DWORD dwValue);
	virtual bool setR(int iX, int iY, int iValue);
	virtual bool setG(int iX, int iY, int iValue);
	virtual bool setB(int iX, int iY, int iValue);

	virtual bool addR(int iX, int iY, int iValue);
	virtual bool addG(int iX, int iY, int iValue);
	virtual bool addB(int iX, int iY, int iValue);

	virtual bool mulR(int iX, int iY, double lfValue);
	virtual bool mulG(int iX, int iY, double lfValue);
	virtual bool mulB(int iX, int iY, double lfValue);

	virtual bool alphaBlend(int iX, int iY, DWORD dwRGB, double lfRate);

	virtual DWORD getColorKey() const;

	virtual bool setColorKey(DWORD dwValue);

	virtual LPDWORD getScanLine(int iY);
	virtual const DWORD * getScanLine(int iY) const;

	virtual bool draw(HDC hdcDst, int iX, int iY) const;
	virtual bool draw(CMDIBSection *lpdisDst, int iX, int iY) const;

	virtual bool line(int iX1, int iY1, int iX2, int iY2, DWORD dwRGB);
	virtual bool rect(int iX1, int iY1, int iX2, int iY2, DWORD dwRGB);
	virtual bool fillRect(int iX1, int iY1, int iX2, int iY2, DWORD dwRGB);
	virtual bool textOut(int iX, int iY, LPCTSTR lpszSrc, DWORD dwRGB);

	virtual HBITMAP getHBITMAP() const;
	virtual HDC getHDC() const;

private:

	bool m_bValid;
	HBRUSH m_brClear;

	int m_iWidth, m_iHeight;
	DWORD m_dwColorKey;
	DWORD m_dwClearBGR;

	HBITMAP m_hbmpBitmap, m_hbmpOld;
	HDC m_hdcBitmap;

	BITMAPINFO m_biHeader;
	LPDWORD m_lppxPixels;

	COLORREF getCOLORREF(DWORD) const;

};
