#pragma once
#include <windows.h>
#include <vector>
#include "MDIBSection.h"

class CDate {

public:

	CDate();
	CDate(const CDate &dtSrc);

	bool assign(const CDate *lpdtSrc);

	int getYear() const;
	int getMonth() const;
	int getDay() const;

	bool setDate(int iYear, int iMonth, int iDay);

	bool setYear(int iYear);
	bool setMonth(int iMonth);
	bool setDay(int iDay);

private:

	int m_iYear, m_iMonth, m_iDay;

};

class CDataItem {

public:

	CDataItem();
	CDataItem(const CDataItem *lpdiSrc);

	bool assign(const CDataItem *lpdiSrc);

	const CDate * getDate() const;

	double getStart() const;
	double getHigh() const;
	double getLow() const;
	double getEnd() const;

	bool setData(int iYear, int iMonth, int iDay, double lfStart, double lfHigh, double lfLow, double lfEnd);
	bool setData(const CDate * lpdtSrc, double lfStart, double lfHigh, double lfLow, double lfEnd);

	bool setDate(int iYear, int iMonth, int iDay);
	bool setDate(const CDate * lpdtSrc);

	bool setStart(double lfStart);
	bool setHigh(double lfHigh);
	bool setLow(double lfLow);
	bool setEnd(double lfEnd);

private:

	CDate m_dtDate;

	double m_lfStart, m_lfHigh, m_lfLow, m_lfEnd;

};

class CDataList {

public:

	CDataList();

	CDataList(const CDataList *lpdlSrc);

	bool assign(const CDataList *lpdlSrc);

	bool add(const CDataItem * lpdiSrc);

	bool clear();

	CDataItem * getItem(int iIndex);
	const CDataItem * getItem(int iIndex) const;

	double getMinimum() const;
	double getMax() const;

	double getMA(int iIndex, int iSamples);

	bool load(LPCTSTR lpszFilePath);

	int getLastIndex() const;

private:

	std::vector<CDataItem *> m_adiItems;

};

class CChartBitmap:public CMDIBSection {

public:

	CChartBitmap();
	bool setData(const CDataList *lpdlSrc);
	bool drawData();

private:

	int m_iStartIndex, m_iLastIndex;
	double m_lfYRangeLow, m_lfYRangeHigh;
	int m_iXSize, m_iYSize;
	int m_iBarWidth;

	CDataList *m_lpdlData;

};
